/* @HEADER@ */
/* ***********************************************************************
// 
//           TSFExtended: Trilinos Solver Framework Extended
//                 Copyright (2004) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//  
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
// Questions? Contact Michael A. Heroux (maherou@sandia.gov) 
// 
// **********************************************************************/
/* @HEADER@ */

#ifndef TSFITERATIVESOLVER_HPP
#define TSFITERATIVESOLVER_HPP

#include "TSFConfigDefs.hpp"
#include "TSFLinearSolverBase.hpp"

namespace TSFExtended
{
  using namespace Teuchos;

  /**
   *
   */
  template <class Scalar>
  class IterativeSolver : public LinearSolverBase<Scalar>
  {
  public:
    /** */
    IterativeSolver(const ParameterList& params = ParameterList());

    /** */
    virtual ~IterativeSolver(){;}
    
    /** */
    int getMaxiters() const 
    {return parameters().template get<int>(maxitersParam());}

    /** */
    Scalar getTol() const 
    {return parameters().template get<double>(tolParam());}

    /** */
    static string maxitersParam() {return "maxiters";}

    /** */
    static string tolParam() {return "tol";}

    /** */
    static const int defaultMaxiters() {return 500;}

    /** */
    static const Scalar defaultTol() {return 1.0e-10;}
  };

  
  template <class Scalar> inline
  IterativeSolver<Scalar>::IterativeSolver(const ParameterList& params)
    : LinearSolverBase<Scalar>(params)
  {;}
  
}

#endif
